/****************************************************************************
** Meta object code from reading C++ file 'session.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kimap-26.04.0/src/session.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'session.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5KIMAP7SessionE_t {};
} // unnamed namespace

template <> constexpr inline auto KIMAP::Session::qt_create_metaobjectdata<qt_meta_tag_ZN5KIMAP7SessionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIMAP::Session",
        "jobQueueSizeChanged",
        "",
        "qsizetype",
        "queueSize",
        "connectionLost",
        "connectionFailed",
        "stateChanged",
        "KIMAP::Session::State",
        "newState",
        "oldState",
        "State",
        "Disconnected",
        "NotAuthenticated",
        "Authenticated",
        "Selected"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'jobQueueSizeChanged'
        QtMocHelpers::SignalData<void(qsizetype)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'connectionLost'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'connectionFailed'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(KIMAP::Session::State, KIMAP::Session::State)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 8, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(11, 11, QMC::EnumFlags{}).add({
            {   12, State::Disconnected },
            {   13, State::NotAuthenticated },
            {   14, State::Authenticated },
            {   15, State::Selected },
        }),
    };
    return QtMocHelpers::metaObjectData<Session, qt_meta_tag_ZN5KIMAP7SessionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KIMAP::Session::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KIMAP7SessionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KIMAP7SessionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5KIMAP7SessionE_t>.metaTypes,
    nullptr
} };

void KIMAP::Session::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Session *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->jobQueueSizeChanged((*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[1]))); break;
        case 1: _t->connectionLost(); break;
        case 2: _t->connectionFailed(); break;
        case 3: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<KIMAP::Session::State>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<KIMAP::Session::State>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Session::*)(qsizetype )>(_a, &Session::jobQueueSizeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)()>(_a, &Session::connectionLost, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)()>(_a, &Session::connectionFailed, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Session::*)(KIMAP::Session::State , KIMAP::Session::State )>(_a, &Session::stateChanged, 3))
            return;
    }
}

const QMetaObject *KIMAP::Session::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIMAP::Session::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KIMAP7SessionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KIMAP::Session::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KIMAP::Session::jobQueueSizeChanged(qsizetype _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KIMAP::Session::connectionLost()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KIMAP::Session::connectionFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KIMAP::Session::stateChanged(KIMAP::Session::State _t1, KIMAP::Session::State _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
