/*
    SPDX-FileCopyrightText: 2009 Andras Mantia <amantia@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "sessionuiproxy.h"

KIMAP::SessionUiProxy::~SessionUiProxy() = default;
